package jmeno;
import gui.BasicApplet;
import gui.parts.CharInfo;
import gui.utils.CharInfoCountPosComparator;
import gui.utils.FibbCode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Locale;
import java.util.PriorityQueue;
import javax.swing.JFrame;
import gui.parts.SHCharInfo;
import gui.parts.Legend;

/**
 * Trida s vizualizaci statickeho Huffmanova kodovani.
 * @author Ondrej Hanousek
 */
public class Test extends BasicApplet {
    
    private static int algNumLines=13;
    private static String[] lines;
    private int state;
    private SHCharInfo[] charArray;
    private ArrayList<SHCharInfo> characters=new ArrayList<SHCharInfo>();
    private PriorityQueue<SHCharInfo> queue;
    private int position=0;
    private int codeCreate=0;
    private SHCharInfo one;
    private SHCharInfo zero;
    private SHCharInfo next;
    private SHCharInfo lastFound;
    private FibbCode len=new FibbCode(5);
    private Legend l=new Legend(SHCharInfo.QUEUE_COLOR, Localisation.getString("shQueueLegend"));
    
    /**
     * Vytvori tridu pro vizualizaci Hufmannova kodovani.
     * Trida {@link gui.Localisation} je nastavena tak, aby
     * pouzivala systemove <PRE>Locales</PRE>. To zajistuje
     * zobrazeni popisku a textu v aplikaci ve spravnem jazyce,
     * pokud je k dispozici.
     */
    public Test() {
    }
    
    /**
     * Vytvori tridu pro vizualizaci Huffmanova kodovani.
     * Trida {@link gui.Localisation} je nastavena tak, aby
     * pouzivala <PRE>Locales</PRE> zadane parametrem. To
     * zajistuje zobrazeni popisku a textu v aplikaci ve spravnem
     * jazyce, pokud je k dispozici.
     * @param l <PRE>Locales </PRE> nastavene pro aplikaci
     */
    public Test(Locale l) {
        Localisation.setLocale(l);
    }
    
    /**
     * Pretizeni abstraktni metody ze tridy {@link gui.BasicApplet },
     * ktera je volana pri kazdem kroku - tj. po akci tlacitka
     * "krok" nebo pravidelne ve stavu "spusteno".
     * @return 0 - v pripade ze jsou mozne dalsi kroky,
     * -1 - kdyz neni dalsi krok mozny.
     */
    public int doNextStep() {
        switch (state) {
            //<editor-fold defaultstate="collapsed" desc=" State==0 - pocty " >
            case 0:
                highlightLine(2);
                String pom=readNext();
                if (pom.compareTo("")==0) {
                    charArray=characters.toArray(new SHCharInfo[characters.size()]);
                    state=1;
                    break;
                }
                
                SHCharInfo tempInfo=new SHCharInfo(pom);
                int place=characters.indexOf(tempInfo);
                if (place!=-1) {
                    characters.get(place).incrementCounter();
                } else {
                    characters.add(tempInfo);
                    tempInfo.setPosition(position++);
                    
                    addPart(tempInfo);
                    
                }
                
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==1 - velikost poctu " >
            case 1:
                highlightLine(3);
                 for (char c : VALID.toCharArray()) {
                    SHCharInfo info=new SHCharInfo(c+"");
                    int index=characters.indexOf(info);
                    int number=0;
                    if (index!=-1) {
                        number=characters.get(index).getCount();
                    }
                    write(len.fibbCode(number)+" ");
                    addOutBits(len.fibbLength(number));
                }
                addOutBits((VALID.length()-charArray.length)*2);
                state=2;
                write("");
                break;
                
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==2 - trideni" >
            case 2:
                highlightLine(4);
                
                Arrays.sort(charArray, new CharInfoCountPosComparator());
                for (int i=0;i<charArray.length;i++){
                    charArray[i].setPosition(i);
                }
                state=3;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==3 - vkladani do fronty" >
            case 3:
                highlightLine(5);
                
                l.moveTo(SHCharInfo.X_OFFSET,SHCharInfo.Y_OFFSET+SHCharInfo.BOX_HEIGHT+10 );
                addPart(l);
                queue=new PriorityQueue<SHCharInfo>(charArray.length, new CharInfoCountPosComparator());
                for (int i=0;i<charArray.length;i++){
                    charArray[i].setInQueue(true);
                    queue.add(charArray[i]);
                }
                state=4;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==4 - podminka while" >
            case 4:
                highlightLine(6);
                if (queue.size()<2) {
                    state=8;
                    break;
                }
                state=5;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==5 - vyber z fronty" >
            case 5:
                highlightLine(7);
                one=queue.poll();
                zero=queue.poll();
                one.setSelected(true);
                zero.setSelected(true);
                state=6;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==6 - vytvoreni noveho uzlu" >
            case 6:
                highlightLine(8);
                next=new SHCharInfo(one,zero);
                addPart(next);
                state=7;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==7 - vlozeni noveho do fronty" >
            case 7:
                highlightLine(9);
                queue.add(next);
                next.setInQueue(true);
                one.setInQueue(false);
                zero.setInQueue(false);
                state=4;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==8 - konec while" >
            case 8:
                highlightLine(10);
                next.setInQueue(false);
                queue.remove(next);
                state=9;
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==9 - ohodnoceni kodama" >
            case 9:
                removePart(l);
                highlightLine(11);
                if (codeCreate>0) charArray[codeCreate-1].setHighlighted(false,true);
                charArray[codeCreate].setHighlighted(true,true);
                codeCreate++;
                if (codeCreate==charArray.length) {
                    state=10;
                    rewindInput();
                    break;
                }
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==10 - vytvoreni vystupu" >
            case 10:
                charArray[charArray.length-1].setHighlighted(false,true);
                highlightLine(12);
                pom=readNext();
                if (pom.compareTo("")==0) {
                    state=11;
                    write("");
                    break;
                }
                int ii=0;
                for (ii=0;ii<charArray.length;ii++) {
                    if (charArray[ii].equals(new SHCharInfo(pom))) break;
                }
                if (lastFound!=null) lastFound.setHighlighted(false,false);
                charArray[ii].setHighlighted(true,false);
                lastFound=charArray[ii];
                write(charArray[ii].getCode());
                addOutBits2(charArray[ii].getCode().length());
                break;
                // </editor-fold>
                //<editor-fold defaultstate="collapsed" desc=" State==11 - konec" >
            case 11:
                if (lastFound!=null) lastFound.setHighlighted(false,false);
                highlightLine(13);
                repaint();
                return -1;
                // </editor-fold>
        }
        repaint();
        return 0;
    }
    
    
    /**
     * Metoda pro kontrolu, jestli ma retezec dva ruzne znaky.
     * Nutne, protoze algoritmus neobsahuje vsechny okrajove
     * podminky.
     * @param s retezec pro zkontrolovani
     * @return platnost retezce
     */
    public boolean isValidInput(String s){
        for (int i=0;i<s.length();i++){
            if (s.charAt(i)!=s.charAt(0)) return true;
        }
        return false;
    }
    
    /**
     * Vraci chybovou hlasku, pokud vstup neobsahuje dva ruzne znaky.
     * @return chybova zprava
     */
    public String getInvalidInputMsg(){
        return Localisation.getString("shInvalidInput");
    }
    
    private void fillAlgLines(){
        lines=new String[algNumLines];
        for (int i=0;i<algNumLines;i++){
            String pom=Localisation.getString("shAlgLine"+(i+1));
            lines[i]=pom;
        }
    }
    
    private String[] getCodelines(){
        if (lines==null) fillAlgLines();
        return lines;
    }
    
    
    /**
     * Tato funkce se vola pri inicializaci appletu prohlizecem.
     * Take je volana z metody main pri spusteni jako aplikace.
     * V teto metode se naplni prislusny {@link gui.AlgorithmPanel}
     * lokalizovanou ({@link gui.Localisation }) verzi algoritmu.
     */
    public void init(){
        
        try {
            Localisation.setLocale(new Locale(getParameter("locale")));
        } catch (NullPointerException e) {
            //  Pouziji se defaultni, zde nic nesmi byt, aby to fungovalo i jako aplikace.
        }
        super.init();
        setCodelines(getCodelines());
        setHeadLine(Localisation.getString("shHeadline"));
        highlightLine(1);
        setTwoCounters(true);
    }
    
    /**
     * Metoda volana pri spousteni jako aplikace.
     * @param args parametr muze byt ve tvaru <PRE>Locale</PRE> pro nastaveni jineho jazyka.
     */
    public static void main(String[] args) {
        Test shApplet;
        if (args.length==1) {
            Localisation.setLocale(new Locale(args[0]));
        }
        shApplet = new Test();
        JFrame sfWindow = new JFrame(Localisation.getString("shTitle"));
        sfWindow.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        sfWindow.setContentPane(shApplet);
        shApplet.init();
        sfWindow.pack();
        sfWindow.setVisible(true);
    }
    
    /**
     * Zavolano z tridy {@link gui.BasicApplet }, pokud se ma obnovit puvodni
     * nastaveni appletu.
     */
    public void reset() {
        super.reset();
        state=0;
        characters=new ArrayList<SHCharInfo>();
        position=0;
        codeCreate=0;
    }
}
